int find_minimum(double *a, double v, int n);
void find_top_two(double *a, double v, int n, int *out);
int compare_function(const void *a,const void *b);
double simpson(double fa, double fb, double fmid, double a, double b);
int i4_max ( int i1, int i2 );
int i4_min ( int i1, int i2 );
void r8vec_print ( int n, double a[], char *title );
void timestamp ( void );
int factorial (int n);