double Hotspot_b(double la, double lo,double la0, double lo0,double p_b,double spot_b,double size, int make_grid ,double theta1, double theta2, double r1, double r2, double lambda0, double phi0,double la_cen,double lo_cen);
double Hotspot_T(double la, double lo,double la0, double lo0,double p_T,double spot_T,double size, int make_grid ,double theta1, double theta2, double r1, double r2, double lambda0, double phi0);
double Uniform_b(double p_bright);
double Uniform_T(double T_bright);
double Two_b(double la, double lo, double p_day, double p_night);
double Two_T(double la, double lo, double p_day, double p_night);
double zhang_2016(double lat, double lon, double zeta, double T_n, double delta_T);
double spherical(double lat, double lon, double *a, int therm_flag);
double kreidberg_2016(double lat, double lon, double insol, double albedo, double redist);
double Louden_2017(double lat, double lon, double insol, double albedo, double redist,double T_int);
double lambertian(double lat, double lon, double insol, double albedo);
double great_circle(double la0,double lo0,double lambda0,double phi0,double r,double theta);
